﻿using System;
using System.Runtime.InteropServices;
//using System.Windows;
using Microsoft.Win32;
using System.Collections.Generic;
using System.Globalization;
using Microsoft.Win32.SafeHandles;

namespace CSharp_DHHalLib_Wrapper_App
{
    #region XslHal class
    /// <summary>    
    /// Wrapper class for the XSL HAL DLL.
    /// </summary>
    /// <remarks>
    /// This class exposes wrapper functions for the XSL HAL DLL interface. By this interface, it is 
    /// possible to access several hardware related functions in an easy way.
    /// </remarks>
    /// 

    public enum DHCOM_GPIOEnum
    {
        DHCOM_GPIO_A = 1,
        DHCOM_GPIO_B,
        DHCOM_GPIO_C,
        DHCOM_GPIO_D,
        DHCOM_GPIO_E,
        DHCOM_GPIO_F,
        DHCOM_GPIO_G,
        DHCOM_GPIO_H,
        DHCOM_GPIO_I,
        DHCOM_GPIO_J,
        DHCOM_GPIO_K,
        DHCOM_GPIO_L,
        DHCOM_GPIO_M,
        DHCOM_GPIO_N,
        DHCOM_GPIO_O,
        DHCOM_GPIO_P,
        DHCOM_GPIO_Q,
        DHCOM_GPIO_R,
        DHCOM_GPIO_S,
        DHCOM_GPIO_T,
        DHCOM_GPIO_U,
        DHCOM_GPIO_V,
        DHCOM_GPIO_W,
        DHCOM_GPIO_UART1_CTS,
        DHCOM_GPIO_UART1_RTS,
        DHCOM_GPIO_UART2_CTS,
        DHCOM_GPIO_UART2_RTS,
        DHCOM_GPIO_UART3_CTS,
        DHCOM_GPIO_UART3_RTS,
        DHCOM_GPIO_UART4_CTS,
        DHCOM_GPIO_UART4_RTS,
        DHCOM_GPIO_UART5_CTS,
        DHCOM_GPIO_UART5_RTS,
        DHCOM_GPIO_NOT_DEFINED
    };

    public enum GPIOConvertType
    {
        DHCOM_GPIO_ID = 0,
        DHCOM_GPIO_PORT,
        DHCOM_GPIO_PIN
    };

    public enum DHCOM_GPIOISRType
    {
        DHCOM_GPIO_INTR_LOW_LEV = 0,
        DHCOM_GPIO_INTR_HIGH_LEV,
        DHCOM_GPIO_INTR_RISE_EDGE,
        DHCOM_GPIO_INTR_FALL_EDGE,
        DHCOM_GPIO_INTR_BOTH_EDGE,
        DHCOM_GPIO_INTR_NONE
    };

    public enum DHCOM_I2CEnum
    {
        DHCOM_I2C0 = 0,
        DHCOM_I2C1,
        DHCOM_I2C2,
        DHCOM_I2C3,
        /*DHCOM_I2C4,*/
        DHCOM_I2C_NOT_DEFINED
    };

    public enum DHCOM_I2CFREQMode
    {
        I2C_100K = 0,
        I2C_400K,
        I2C_800K,
        I2C_1600K,
        I2C_2400K,
        I2C_3200K
    };

    public enum DHCOM_SPIEnum
    {
	    DHCOM_SPI1=0,
	    DHCOM_SPI2,
	    DHCOM_SPI_NOT_DEFINED
    };

    public enum DHCOM_SPIModeEnum
    {
	    SPI_CS0=0,
	    SPI_CS1,
	    SPI_CS2,
	    SPI_CS3
    };

    public enum DHCOM_PHYEnum
    {
	    DHCOM_PHY1=1,
	    DHCOM_PHY2,
	    DHCOM_PHY3,
	    DHCOM_PHY4,
	    DHCOM_PHY5,
	    DHCOM_PHY6,
	    DHCOM_PHY7,
	    DHCOM_PHY8,
	    DHCOM_PHY9
    };

    public enum DHCOM_UARTEnum
    {
	    DHCOM_UART1=1,
	    DHCOM_UART2,
	    DHCOM_UART3
    };

    public enum DHCOM_UARTModeEnum
    {
	    DHCOM_HSHAKE_OFF=1,
	       DHCOM_HSHAKE_SOFTWARE,
	    DHCOM_HSHAKE_HARDWARE
    };

    public enum DHCOM_UARTParityEnum
    {
	    DHCOM_NOPARITY=0,
	    DHCOM_ODDPARITY,
	    DHCOM_EVENPARITY,
	    DHCOM_MARKPARITY,
	    DHCOM_SPACEPARITY
    };

    public enum DHCOM_UARTStopBitEnum
    {
	    DHCOM_ONESTOPBIT=0,
	    DHCOM_ONE5STOPBITS,
	    DHCOM_TWOSTOPBITS
       };

    public enum DHCOM_UARTByteSizeEnum
    {
	    DHCOM_5BIT=5,
	    DHCOM_6BIT,
	    DHCOM_7BIT,
	    DHCOM_8BIT
    };

    public enum DHCOM_UARTWEvent
    {
	    SIG_RX_CHAR_DETECT_Flag=1,
	    SIG_RX_EVENT_DETECT_Flag=2,
	    SIG_TX_BUFFER_EMPTY_Flag=4,
	    SIG_CTS_CHANGED_Flag=8,
	    SIG_DSR_CHANGED_Flag=16,
	    SIG_RLSD_CHANGED_Flag=32,
	    SIG_BREAK_DETECT_Error=64,
	    SIG_LINE_STATUS_Error=128,
	    SIG_RING_DETECT_Error=512
    };

    public enum DHCOM_CANEnum
    {
	    DHCOM_CAN1=1
    };

    public enum DHCOM_CANModeEnum
    {
	    DHCOM_CAN_STANDARD=0,
	    DHCOM_CAN_EXTENDED
    };

    public enum DHCOM_CANFrameEnum
    {
	    DHCOM_CAN_DATA=0,
	    DHCOM_CAN_REMOTE
    };

    public static class DHHalLib
    {
        #region P/Invoke declarations for VITA HAL DLL interface

        const string LibName = "DHHalLib.dll";

        // Version
        [DllImport(LibName, EntryPoint = "DHHalLibGetVersion")]
        public static extern UInt32 DHHalLibGetVersion();

        [DllImport(LibName, EntryPoint = "WinCEImageGetVersion")]
        public static extern UInt32 WinCEImageGetVersion();

        [DllImport(LibName, EntryPoint = "BSPGetVersion")]
        public static extern UInt32 BSPGetVersion();

        // GPIO
        [DllImport(LibName, EntryPoint = "GPIOSetDirection")]
        public static extern bool GPIOSetDirection(DHCOM_GPIOEnum eGpio, [MarshalAs(UnmanagedType.U1)] bool bInOut, [MarshalAs(UnmanagedType.U1)] bool bDefaultState);

        [DllImport(LibName, EntryPoint = "GPIOSetPin")]
        public static extern void GPIOSetPin(DHCOM_GPIOEnum eGpio, [MarshalAs(UnmanagedType.U1)] bool bState);	            // Sets the GPIO Pin state

        [DllImport(LibName, EntryPoint = "GPIOGetPin")]
        public static extern bool GPIOGetPin(DHCOM_GPIOEnum eGpio);	                            // Returns the GPIO Pin state

        [DllImport(LibName, EntryPoint = "GPIOSetInterrupt")]
        public static extern bool GPIOSetInterrupt(DHCOM_GPIOEnum eGpio, DHCOM_GPIOISRType eISRType, SafeWaitHandle hEvent);

        [DllImport(LibName, EntryPoint = "GPIOClearInterrupt")]
        public static extern bool GPIOClearInterrupt(DHCOM_GPIOEnum eGpio);

        // I2C
        [DllImport(LibName, EntryPoint = "I2COpen")]
        public static extern bool I2COpen(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq);					// Open the I2C bus

        [DllImport(LibName, EntryPoint = "I2CClose")]                // Close the I2C bus
        public static extern void I2CClose(DHCOM_I2CEnum eI2CPort);

        [DllImport(LibName, EntryPoint = "I2CRead")]
        public static extern bool I2CRead(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, byte[] pOutBuffer);					// Reads from the I2C bus

        [DllImport(LibName, EntryPoint = "I2CWrite")]
        public static extern bool I2CWrite(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, byte cValue);	// Write to the I2C bus

        [DllImport(LibName, EntryPoint = "I2CReadMultipleBytes")]
        public static extern bool I2CReadMultipleBytes(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, byte cBytes, byte[] pOutBuffer); // Read Multiple Bytes from the Spi bus

        [DllImport(LibName, EntryPoint = "I2CWriteMultipleBytes")]
        public static extern bool I2CWriteMultipleBytes(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, byte cBytes, byte[] pOutBuffer); // Write Multiple Bytes to the Spi bus

        // Spi
        [DllImport(LibName, EntryPoint = "SPIOpen")]
        public static extern bool SPIOpen(DHCOM_SPIEnum eSPIPort);					// Open the I2C bus

        [DllImport(LibName, EntryPoint = "SPIClose")]                // Close the SPI bus
        public static extern void SPIClose(DHCOM_SPIEnum eSPIPort);

        [DllImport(LibName, EntryPoint = "SPIConfigure")]                // Configure the SPI bus
        public static extern bool SPIConfigure(DHCOM_SPIEnum eSPIPort, DHCOM_SPIModeEnum eSPIMode, uint iSPIFreq, [MarshalAs(UnmanagedType.U1)] bool useDMA);

        [DllImport(LibName, EntryPoint = "SPIExchange")]                // Exchange data to/from the SPI bus
        public static extern bool SPIExchange(DHCOM_SPIEnum eSPIPort, int iBitCount, ulong[] pOutBuffer, ulong[] pInBuffer);

        // Can
        [DllImport(LibName, EntryPoint = "CANOpen")]
        public static extern bool CANOpen(DHCOM_CANEnum eCANPort, DHCOM_CANModeEnum eCANMode);					// Open the Can(x) bus

        [DllImport(LibName, EntryPoint = "CANClose")]                // Close the Can(x) bus
        public static extern void CANClose(DHCOM_CANEnum eCANPort);

        [DllImport(LibName, EntryPoint = "CANConfigureMode")]
        public static extern bool CANConfigureMode(DHCOM_CANEnum eCANPort, DHCOM_CANModeEnum eCANMode, UInt32 lCANFreq = 0, [MarshalAs(UnmanagedType.U1)] bool bCANTLPrio = false, [MarshalAs(UnmanagedType.U1)] bool bCANLoopback = false);					// Configure Can(x) Mode bus

        [DllImport(LibName, EntryPoint = "CANConfigureTiming")]
        public static extern bool CANConfigureTiming(DHCOM_CANEnum eCANPort, UInt32 lPrescaler, byte cPropSeg, byte cPhase1Seg, byte cPhase2Seg, byte RJW);	// Configure Can(x) Timeout bus

        [DllImport(LibName, EntryPoint = "CANConfigureFilter")]
        public static extern bool CANConfigureFilter(DHCOM_CANEnum eCANPort, int iFilterID, int iFilterIDMask, [MarshalAs(UnmanagedType.U1)] bool bFilterFormat, [MarshalAs(UnmanagedType.U1)] bool bFilterRemote); // Configure Can(x) Filter bus

        [DllImport(LibName, EntryPoint = "CANResetFilter")]
        public static extern bool CANResetFilter(DHCOM_CANEnum eCANPort); // Reset Can(x) Filter bus

        [DllImport(LibName, EntryPoint = "CANConfigureMessage")]
        public static extern bool CANConfigureMessage(DHCOM_CANEnum eCANPort, DHCOM_CANFrameEnum eCANFrame, int iCANMsgID, byte cCANMsgPrio, UInt32 lCANMsgTimeout, [MarshalAs(UnmanagedType.U1)] bool bReadWrite); // Configure Can(x) Message

        [DllImport(LibName, EntryPoint = "CANGetMessageParameter")]
        public static extern bool CANGetMessageParameter(DHCOM_CANEnum eCANPort, ref int iCANMsgID, ref int iCANMsgLength, ref int iCANFrame, ref byte cCANMsgPrio, ref UInt32 lCANMsgTimeout, ref int iTXAbort, [MarshalAs(UnmanagedType.U1)] bool bReadWrite); // Get Can(x) Message Header Informations

        [DllImport(LibName, EntryPoint = "CANRead")]
        public static extern bool CANRead(DHCOM_CANEnum eCANPort, byte[] pInBuffer, int iDataLength, ref int iResult); // Read Single/Multiple Bytes from the Can(x) bus

        [DllImport(LibName, EntryPoint = "CANWrite")]
        public static extern bool CANWrite(DHCOM_CANEnum eCANPort, byte[] pOutBuffer, int iDataLength, ref int iResult); // Write Single/Multiple Bytes to the Can(x) bus

        //Physical Address
        [DllImport(LibName, EntryPoint = "ReadPhysicalAddress")]
        public static extern int ReadPhysicalAddress(int iPhyAddr, byte cRegSize);                         		// Reads from Memory (I/O)

        [DllImport(LibName, EntryPoint = "WritePhysicalAddress")]
        public static extern int WritePhysicalAddress(int iPhyAddr, byte cRegSize, int iValue, [MarshalAs(UnmanagedType.U1)] bool bReadBack = false);	// Write to Memory (I/O)

        // Backlight
        [DllImport(LibName, EntryPoint = "SetBacklightLevel")]
        public static extern bool SetBacklightLevel(int iLevel);

        [DllImport(LibName, EntryPoint = "SetBacklightSuspend")]
        public static extern bool SetBacklightSuspend(int Timeout, [MarshalAs(UnmanagedType.U1)] bool bEnable);

        // Touch
        [DllImport(LibName, EntryPoint = "CalibrateTouch")]
        public static extern bool CalibrateTouch();

        [DllImport(LibName, EntryPoint = "ClearTouchCalibrationData")]
        public static extern bool ClearTouchCalibrationData();

        // Reset
        [DllImport(LibName, EntryPoint = "SoftReset")]
        public static extern void SoftReset();

        // Registry
        [DllImport(LibName, EntryPoint = "SaveRegistry")]
        public static extern void SaveRegistry();

        [DllImport(LibName, EntryPoint = "CreateRegistryBackup")]
        public static extern bool CreateRegistryBackup();

        [DllImport(LibName, EntryPoint = "RestoreRegistryBackup")]
        public static extern bool RestoreRegistryBackup();
    }  
#endregion
}
#endregion
